package org.seamframework.sql;

import java.util.Collections;
import java.util.List;

public class From<X> {

	private final class ColumnRef<Y> implements Expression<Y> {
		private final Column<X, Y> column;

		private ColumnRef(Column<X, Y> column) {
			this.column = column;
		}

		@Override
		public Class<Y> getType() {
			return column.getType();
		}

		@Override
		public String toSql() {
			return alias==null ? 
					column.getName() : 
					alias + '.' + column.getName();
		}
		@Override @SuppressWarnings("unchecked")
		public List<Parameter<?>> getParameters() {
			return Collections.EMPTY_LIST;
		}
	}

	private final Table<X> table;
	private final String alias;
	
	public From(Table<X> table, String alias) {
		this.table = table;
		this.alias = alias;
	}
	
	public From(Table<X> table) {
		this(table, null);
	}
	
	public <Y> Expression<Y> get(final Column<X, Y> column) {
		return new ColumnRef<Y>(column);
	}
	
	public String toSql() {
		return alias==null ? 
				table.getQualifiedName() :
				table.getQualifiedName() + " as " + alias;
	}
		
}
